@@ -2,7 +2,7 @@ import json
 from os import getenv
 from pathlib import Path
 from textwrap import dedent
-from typing import Dict, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 from agno.agent import Agent
 from agno.team import Team
@@ -24,7 +24,6 @@ except ImportError:
 DEFAULT_INSTRUCTIONS = dedent(
     """\
     You have access to a persistent Daytona sandbox for code execution. The sandbox maintains state across interactions.
-    
     Available tools:
     - `run_code`: Execute code in the sandbox
     - `run_shell_command`: Execute shell commands (bash)
@@ -33,21 +32,19 @@ DEFAULT_INSTRUCTIONS = dedent(
     - `list_files`: List directory contents
     - `delete_file`: Delete files or directories
     - `change_directory`: Change the working directory
-    
     MANDATORY: When users ask for code (Python, JavaScript, TypeScript, etc.), you MUST:
     1. Write the code
     2. Execute it using run_code tool
     3. Show the actual output/results
     4. Never just provide code without executing it
-    
     CRITICAL WORKFLOW:
     1. Before running Python scripts, check if required packages are installed
     2. Install missing packages with: run_shell_command("pip install package1 package2")
     3. When running scripts, capture both output AND errors
     4. If a script produces no output, check for errors or add print statements
-    
+
     IMPORTANT: Always use single quotes for the content parameter when creating files
-    
+
     Remember: Your job is to provide working, executed code examples, not just code snippets!
     """
 )
@@ -110,18 +107,18 @@ class DaytonaTools(Toolkit):
         )
 
         self.daytona = Daytona(self.config)
-
+        tools: List[Any] = [
+            self.run_code,
+            self.run_shell_command,
+            self.create_file,
+            self.read_file,
+            self.list_files,
+            self.delete_file,
+            self.change_directory,
+        ]
         super().__init__(
             name="daytona_tools",
-            tools=[
-                self.run_code,
-                self.run_shell_command,
-                self.create_file,
-                self.read_file,
-                self.list_files,
-                self.delete_file,
-                self.change_directory,
-            ],
+            tools=tools,
             instructions=self.instructions,
             add_instructions=add_instructions,
             **kwargs,
