@@ -94,27 +94,29 @@ class GoogleSheetsTools(Toolkit):
         creds: Optional[Credentials] = None,
         creds_path: Optional[str] = None,
         token_path: Optional[str] = None,
-        read: bool = True,
-        create: bool = False,
-        update: bool = False,
-        duplicate: bool = False,
         oauth_port: int = 0,
+        enable_read_sheet: bool = True,
+        enable_create_sheet: bool = False,
+        enable_update_sheet: bool = False,
+        enable_create_duplicate_sheet: bool = False,
+        all: bool = False,
         **kwargs,
     ):
         """Initialize GoogleSheetsTools with the specified configuration.
 
         Args:
-            scopes (Optional[List[str]]): Custom OAuth scopes. If None, determined by operations.
+            scopes (Optional[List[str]]): Custom OAuth scopes. If None, uses write scope by default.
             spreadsheet_id (Optional[str]): ID of the target spreadsheet.
             spreadsheet_range (Optional[str]): Range within the spreadsheet.
             creds (Optional[Credentials]): Pre-existing credentials.
             creds_path (Optional[str]): Path to credentials file.
             token_path (Optional[str]): Path to token file.
-            read (bool): Enable read operations. Defaults to True.
-            create (bool): Enable create operations. Defaults to False.
-            update (bool): Enable update operations. Defaults to False.
-            duplicate (bool): Enable duplicate operations. Defaults to False.
             oauth_port (int): Port to use for OAuth authentication. Defaults to 0.
+            enable_read_sheet (bool): Enable reading from a sheet.
+            enable_create_sheet (bool): Enable creating a sheet.
+            enable_update_sheet (bool): Enable updating a sheet.
+            enable_create_duplicate_sheet (bool): Enable creating a duplicate sheet.
+            all (bool): Enable all tools.
         """
 
         self.spreadsheet_id = spreadsheet_id
@@ -128,19 +130,21 @@ class GoogleSheetsTools(Toolkit):
         # Determine required scopes based on operations if no custom scopes provided
         if scopes is None:
             self.scopes = []
-            if read:
+            if enable_read_sheet:
                 self.scopes.append(self.DEFAULT_SCOPES["read"])
-            if create or update or duplicate:
+            if enable_create_sheet or enable_update_sheet or enable_create_duplicate_sheet:
                 self.scopes.append(self.DEFAULT_SCOPES["write"])
             # Remove duplicates while preserving order
             self.scopes = list(dict.fromkeys(self.scopes))
         else:
             self.scopes = scopes
             # Validate that required scopes are present for requested operations
-            if (create or update or duplicate) and self.DEFAULT_SCOPES["write"] not in self.scopes:
+            if (enable_create_sheet or enable_update_sheet or enable_create_duplicate_sheet) and self.DEFAULT_SCOPES[
+                "write"
+            ] not in self.scopes:
                 raise ValueError(f"The scope {self.DEFAULT_SCOPES['write']} is required for write operations")
             if (
-                read
+                enable_read_sheet
                 and self.DEFAULT_SCOPES["read"] not in self.scopes
                 and self.DEFAULT_SCOPES["write"] not in self.scopes
             ):
@@ -149,13 +153,13 @@ class GoogleSheetsTools(Toolkit):
                 )
 
         tools: List[Any] = []
-        if read:
+        if all or enable_read_sheet:
             tools.append(self.read_sheet)
-        if create:
+        if all or enable_create_sheet:
             tools.append(self.create_sheet)
-        if update:
+        if all or enable_update_sheet:
             tools.append(self.update_sheet)
-        if duplicate:
+        if all or enable_create_duplicate_sheet:
             tools.append(self.create_duplicate_sheet)
 
         super().__init__(name="google_sheets_tools", tools=tools, **kwargs)
