@@ -6,13 +6,20 @@ from agno.utils.log import log_debug, log_info, logger
 
 
 class ShellTools(Toolkit):
-    def __init__(self, base_dir: Optional[Union[Path, str]] = None, **kwargs):
+    def __init__(
+        self,
+        base_dir: Optional[Union[Path, str]] = None,
+        enable_run_shell_command: bool = True,
+        all: bool = False,
+        **kwargs,
+    ):
         self.base_dir: Optional[Path] = None
         if base_dir is not None:
             self.base_dir = Path(base_dir) if isinstance(base_dir, str) else base_dir
 
         tools = []
-        tools.append(self.run_shell_command)
+        if all or enable_run_shell_command:
+            tools.append(self.run_shell_command)
 
         super().__init__(name="shell_tools", tools=tools, **kwargs)
 
