@@ -1,3 +1,4 @@
+from enum import Enum
 from typing import Any, Dict, Optional, Union, get_args, get_origin
 
 from pydantic import BaseModel
@@ -140,11 +141,16 @@ def get_json_schema_for_arg(type_hint: Any) -> Optional[Dict[str, Any]]:
                     continue
             return {"anyOf": types} if types else None
 
-    elif issubclass(type_hint, BaseModel):
+    if isinstance(type_hint, type) and issubclass(type_hint, Enum):
+        enum_values = [member.value for member in type_hint]
+        return {"type": "string", "enum": enum_values}
+
+    if isinstance(type_hint, type) and issubclass(type_hint, BaseModel):
         # Get the schema and inline it
         schema = type_hint.model_json_schema()
         return inline_pydantic_schema(schema)  # type: ignore
-    elif hasattr(type_hint, "__dataclass_fields__"):
+
+    if hasattr(type_hint, "__dataclass_fields__"):
         # Convert dataclass to JSON schema
         properties = {}
         required = []
